CREATE PROCEDURE [dbo].[sp_database_rebuild]
AS 
BEGIN

BEGIN TRY
DECLARE @ujAdatbazis varchar(30), @tablaNev varchar(8), @oszlopNev varchar(10), @constraintNev varchar(50), @defaultValue varchar(50), @command nvarchar(4000),
	    @compatibility_level tinyint, @collation_name varchar(100)

SET @ujAdatbazis = DB_NAME() + '_teszt_uj'

SELECT @compatibility_level = compatibility_level, @collation_name = collation_name FROM sys.databases WHERE name = DB_NAME()

DECLARE @mdfName varchar(30), @ndfName varchar(30), @ldfName varchar(30)
DECLARE @mdfFileName varchar(30), @ndfFileName varchar(30), @ldfFileName varchar(30)
DECLARE @mdfPath varchar(300), @ndfPath varchar(300), @ldfPath varchar(300)
DECLARE @mdfSorszam int, @ndfSorszam int, @ldfSorszam int

SELECT @mdfName = name, @mdfPath = physical_name FROM sys.master_files WHERE database_id = DB_ID() AND physical_name LIKE '%.mdf'
SELECT @ndfName = name, @ndfPath = physical_name FROM sys.master_files WHERE database_id = DB_ID() AND physical_name LIKE '%.ndf'
SELECT @ldfName = name, @ldfPath = physical_name FROM sys.master_files WHERE database_id = DB_ID() AND physical_name LIKE '%.ldf'

SET @mdfFileName = RIGHT(@mdfPath, CHARINDEX('\',REVERSE(@mdfPath))-1)
SET @ndfFileName = RIGHT(@ndfPath, CHARINDEX('\',REVERSE(@ndfPath))-1)
SET @ldfFileName = RIGHT(@ldfPath, CHARINDEX('\',REVERSE(@ldfPath))-1)

IF PATINDEX('%[0-9]%',@mdfFileName) > 0
BEGIN
	SET @mdfSorszam = CAST(SUBSTRING(@mdfFileName, PATINDEX('%[0-9]%',@mdfFileName), LEN(@mdfFileName) - PATINDEX('%[0-9]%',REVERSE(@mdfFileName)) - PATINDEX('%[0-9]%',@mdfFileName) + 2) AS int)
	SET @mdfPath = REPLACE(@mdfPath, CAST(@mdfSorszam AS varchar(4))+'.mdf',CAST((@mdfSorszam+1) AS varchar(4))+'.mdf')
END
ELSE
BEGIN
	SET @mdfPath = REPLACE(@mdfPath, '.mdf','1.mdf')
END

IF PATINDEX('%[0-9]%',@ndfFileName) > 0
BEGIN
	SET @ndfSorszam = CAST(SUBSTRING(@ndfFileName, PATINDEX('%[0-9]%',@ndfFileName), LEN(@ndfFileName) - PATINDEX('%[0-9]%',REVERSE(@ndfFileName)) - PATINDEX('%[0-9]%',@ndfFileName) + 2) AS int)
	SET @ndfPath = REPLACE(@ndfPath, CAST(@ndfSorszam AS varchar(4))+'.ndf',CAST((@ndfSorszam+1) AS varchar(4))+'.ndf')
END
ELSE
BEGIN
	SET @ndfPath = REPLACE(@ndfPath, '.ndf','1.ndf')
END

IF PATINDEX('%[0-9]%',@ldfFileName) > 0
BEGIN
	SET @ldfSorszam = CAST(SUBSTRING(@ldfFileName, PATINDEX('%[0-9]%',@ldfFileName), LEN(@ldfFileName) - PATINDEX('%[0-9]%',REVERSE(@ldfFileName)) - PATINDEX('%[0-9]%',@ldfFileName) + 2) AS int)
	SET @ldfPath = REPLACE(@ldfPath, CAST(@ldfSorszam AS varchar(4))+'.ldf',CAST((@ldfSorszam+1) AS varchar(4))+'.ldf')
END
ELSE
BEGIN
	SET @ldfPath = REPLACE(@ldfPath, '.ldf','1.ldf')
END

SET @command = 'CREATE DATABASE ['+@ujAdatbazis+']
	CONTAINMENT = NONE
	ON PRIMARY 
	(NAME = '''+@mdfName+''', 
	FILENAME = '''+@mdfPath+''', MAXSIZE = UNLIMITED, FILEGROWTH = 1024KB), 
	FILEGROUP [DOKUMENT]
	(NAME = '''+@ndfName+''', 
	FILENAME = '''+@ndfPath+''', MAXSIZE = UNLIMITED, FILEGROWTH = 1024KB)
	LOG ON 
	(NAME = '''+@ldfName+''', 
	FILENAME = '''+@ldfPath+''', MAXSIZE = 2048GB, FILEGROWTH = 10%)
	COLLATE '+@collation_name

EXEC (@command)

SET @command = 'ALTER DATABASE ['+@ujAdatbazis+'] SET COMPATIBILITY_LEVEL = '+CAST(@compatibility_level AS varchar)
EXEC (@command)

IF CURSOR_STATUS('global','TablaCursor')>=-1
BEGIN
	CLOSE TablaCursor
	DEALLOCATE TablaCursor
END

DECLARE TablaCursor CURSOR FOR

SELECT TABLE_NAME
FROM INFORMATION_SCHEMA.TABLES
WHERE TABLE_TYPE = 'BASE TABLE' AND TABLE_CATALOG = DB_NAME()
ORDER BY TABLE_NAME

OPEN TablaCursor

FETCH NEXT FROM TablaCursor INTO @tablaNev
WHILE @@FETCH_STATUS = 0
BEGIN
	PRINT @tablaNev

	SET @command = 'SELECT * INTO '+@ujAdatbazis+'.dbo.'+@tablaNev+' FROM '+@tablaNev
	EXEC (@command)

	FETCH NEXT FROM TablaCursor INTO @tablaNev
END
CLOSE TablaCursor
DEALLOCATE TablaCursor

--Primary key
IF CURSOR_STATUS('global','PKCursor')>=-1
BEGIN
	CLOSE PKCursor
	DEALLOCATE PKCursor
END

DECLARE PKCursor CURSOR FOR

SELECT KCU.TABLE_NAME, KCU.CONSTRAINT_NAME, KCU.COLUMN_NAME
FROM INFORMATION_SCHEMA.TABLE_CONSTRAINTS AS TC
	 INNER JOIN INFORMATION_SCHEMA.KEY_COLUMN_USAGE AS KCU ON TC.CONSTRAINT_TYPE = 'PRIMARY KEY' AND TC.CONSTRAINT_NAME = KCU.CONSTRAINT_NAME 
ORDER BY KCU.TABLE_NAME, KCU.COLUMN_NAME

OPEN PKCursor

FETCH NEXT FROM PKCursor INTO @tablaNev, @constraintNev, @oszlopNev
WHILE @@FETCH_STATUS = 0
BEGIN
	PRINT @constraintNev

	SET @command = 'ALTER TABLE '+@ujAdatbazis+'.dbo.'+@tablaNev+'
					ADD CONSTRAINT '+@constraintNev+' PRIMARY KEY CLUSTERED ('+@oszlopNev+')'

	EXEC (@command)

	FETCH NEXT FROM PKCursor INTO @tablaNev, @constraintNev, @oszlopNev
END
CLOSE PKCursor
DEALLOCATE PKCursor

--Constraints
IF CURSOR_STATUS('global','ConCursor')>=-1
BEGIN
	CLOSE ConCursor
	DEALLOCATE ConCursor
END

DECLARE ConCursor CURSOR FOR

SELECT o.name, con.name, con.definition, col.name
FROM sys.default_constraints con
	 LEFT JOIN sys.objects o ON con.parent_object_id = o.object_id
	 LEFT JOIN sys.all_columns col ON con.parent_column_id = col.column_id AND con.parent_object_id = col.object_id
ORDER BY o.name, col.name

OPEN ConCursor

FETCH NEXT FROM ConCursor INTO @tablaNev, @constraintNev, @defaultValue, @oszlopNev
WHILE @@FETCH_STATUS = 0
BEGIN
	PRINT @constraintNev

	SET @command = 'ALTER TABLE '+@ujAdatbazis+'.dbo.'+@tablaNev+'
					ADD CONSTRAINT '+@constraintNev+' DEFAULT '+@defaultValue+' FOR '+@oszlopNev

	EXEC (@command)

	FETCH NEXT FROM ConCursor INTO @tablaNev, @constraintNev, @defaultValue, @oszlopNev
END
CLOSE ConCursor
DEALLOCATE ConCursor

--Function
IF CURSOR_STATUS('global','FunctionCursor')>=-1
BEGIN
	CLOSE FunctionCursor
	DEALLOCATE FunctionCursor
END

DECLARE FunctionCursor CURSOR FOR

SELECT ROUTINE_NAME, 'EXEC '+@ujAdatbazis+'.sys.sp_executesql N'''+REPLACE(ROUTINE_DEFINITION,'''','''''')+''''
FROM INFORMATION_SCHEMA.ROUTINES
WHERE ROUTINE_TYPE = 'FUNCTION'
ORDER BY ROUTINE_NAME

OPEN FunctionCursor

FETCH NEXT FROM FunctionCursor INTO @constraintNev, @command
WHILE @@FETCH_STATUS = 0
BEGIN
	PRINT @constraintNev
	EXEC (@command)

	FETCH NEXT FROM FunctionCursor INTO @constraintNev, @command
END
CLOSE FunctionCursor
DEALLOCATE FunctionCursor

--View
IF CURSOR_STATUS('global','ViewCursor')>=-1
BEGIN
	CLOSE ViewCursor
	DEALLOCATE ViewCursor
END

DECLARE ViewCursor CURSOR FOR

SELECT TABLE_NAME, 'EXEC '+@ujAdatbazis+'.sys.sp_executesql N'''+REPLACE(VIEW_DEFINITION,'''','''''')+''''
FROM INFORMATION_SCHEMA.VIEWS
ORDER BY CASE WHEN TABLE_NAME = 'csakfiz' THEN 'zs1' WHEN TABLE_NAME = 'szallcim' THEN 'zs2' ELSE TABLE_NAME END

OPEN ViewCursor

FETCH NEXT FROM ViewCursor INTO @constraintNev, @command
WHILE @@FETCH_STATUS = 0
BEGIN
	PRINT @constraintNev
	EXEC sys.sp_executesql @command;

	FETCH NEXT FROM ViewCursor INTO @constraintNev, @command
END
CLOSE ViewCursor
DEALLOCATE ViewCursor

--Stored procedure
IF CURSOR_STATUS('global','SPCursor')>=-1
BEGIN
	CLOSE SPCursor
	DEALLOCATE SPCursor
END

DECLARE SPCursor CURSOR FOR

SELECT ROUTINE_NAME, 'EXEC '+@ujAdatbazis+'.sys.sp_executesql N'''+REPLACE(ROUTINE_DEFINITION,'''','''''')+''''
FROM INFORMATION_SCHEMA.ROUTINES
WHERE ROUTINE_TYPE = 'PROCEDURE' AND ROUTINE_NAME IN('sp_usetup_select','sp_setup_select')
ORDER BY ROUTINE_NAME

OPEN SPCursor

FETCH NEXT FROM SPCursor INTO @constraintNev, @command
WHILE @@FETCH_STATUS = 0
BEGIN
	PRINT @constraintNev
	EXEC (@command)

	FETCH NEXT FROM SPCursor INTO @constraintNev, @command
END
CLOSE SPCursor
DEALLOCATE SPCursor

--Computed columns drop
IF CURSOR_STATUS('global','CCCursor')>=-1
BEGIN
	CLOSE CCCursor
	DEALLOCATE CCCursor
END

DECLARE CCCursor CURSOR FOR

SELECT 'ALTER TABLE '+@ujAdatbazis+'dbo.'+object_name(c.object_id)+' DROP COLUMN '+c.name
FROM sys.computed_columns c
	 INNER JOIN sys.objects o on o.object_id = c.object_id
ORDER BY object_name(c.object_id), c.name

OPEN CCCursor

FETCH NEXT FROM CCCursor INTO @command
WHILE @@FETCH_STATUS = 0
BEGIN
	PRINT @command
	EXEC (@command)

	FETCH NEXT FROM CCCursor INTO @command
END
CLOSE CCCursor
DEALLOCATE CCCursor

PRINT 'Ksz'

END TRY
BEGIN CATCH
	IF CURSOR_STATUS('global','TablaCursor')>=-1
	BEGIN
		CLOSE TablaCursor
		DEALLOCATE TablaCursor
	END

	IF CURSOR_STATUS('global','PKCursor')>=-1
	BEGIN
		CLOSE PKCursor
		DEALLOCATE PKCursor
	END

	IF CURSOR_STATUS('global','ConCursor')>=-1
	BEGIN
		CLOSE ConCursor
		DEALLOCATE ConCursor
	END

	IF CURSOR_STATUS('global','FunctionCursor')>=-1
	BEGIN
		CLOSE FunctionCursor
		DEALLOCATE FunctionCursor
	END

	IF CURSOR_STATUS('global','ViewCursor')>=-1
	BEGIN
		CLOSE ViewCursor
		DEALLOCATE ViewCursor
	END

	IF CURSOR_STATUS('global','SPCursor')>=-1
	BEGIN
		CLOSE SPCursor
		DEALLOCATE SPCursor
	END

	IF CURSOR_STATUS('global','CCCursor')>=-1
	BEGIN
		CLOSE CCCursor
		DEALLOCATE CCCursor
	END

	SELECT ERROR_MESSAGE() AS 'hiba'
END CATCH
END